package rationals

import collection.mutable.Stack
import org.scalatest._
import org.scalatest.prop.PropertyChecks
import org.scalacheck._

class RationalSpec extends FlatSpec with Matchers with PropertyChecks {
  val numbers = Gen.choose(-1000,1000)

  val rationals = for {
    numer <- numbers
    denom <- numbers.filter(_ != 0)
  } yield Rational(numer,denom)

  "rationals" should "not change the denominator when adding integers" in {
    forAll (rationals, numbers) { (rat,nat) =>
      (rat + nat).denom should be (rat.denom)
    }
  }

  they should "yield the same result for r + r as for r * 2" in {
    forAll (rationals) { rat =>
      (rat + rat) should be (rat * 2)
    }
  }

  they should "throw an exception when initialized with denominator 0" in {
    forAll (numbers) { numer =>
      an [IllegalArgumentException] should be thrownBy Rational(numer,0)
    }
  }

  they should "not throw an exception in all other cases" in {
    forAll { (n: Int, d: Int) => whenever (d != 0) {
      noException should be thrownBy Rational(n,d)
    } }
  }
}